
/*********************************************************************
*                                                                    *
*                       Software License Agreement                   *
*                                                                    *
*   The software supplied herewith by Microchip Technology           *
*   Incorporated (the "Company") for its PICmicro Microcontroller   *
*   is intended and supplied to you, the Companys customer, for use *
*   solely and exclusively on Microchip PICmicro Microcontroller     *
*   products. The software is owned by the Company and/or its        *
*   supplier, and is protected under applicable copyright laws. All  *
*   rights are reserved. Any use in violation of the foregoing       *
*   restrictions may subject the user to criminal sanctions under    *
*   applicable laws, as well as to civil liability for the breach of *
*   the terms and conditions of this license.                        *
*                                                                    *
*   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           *
*   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    *
*   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    *
*   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     *
*   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  *
*   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  *
*                                                                    *
*********************************************************************/

/*********************************************************************
*                                                                    *
*   Filename:	    i2c_comm.h                                       *
*   Date:           06/09/2000                                       *
*   Revision:       1.00                                             *
*                                                                    *
*   Tools:          MPLAB 5.00.00                                    *
*                   Hi-Tech PIC C Compiler V7.85                     *
*                                                                    *
*********************************************************************/



#define MaxNumberI2CSlaves    12         // maximum number of I2C slave devices
#define OperNumberI2CSlaves   12         // operational number of I2C slaves

#define MaxChannelsPerSlave   10         // maximum channels of data per slave
#define OperChannelsPerSlave   3         // operational number of channels of data

#define  MaxSlaveRetry   1               // default is one retry

#define  error_mask  0b0000000000000001


#define COMM_STAT  0
#define STATUS0    1
#define STATUS1    2

#define TEMP0      3
#define ADRES0     4
#define ADRES1     5
#define ADRES2     6
#define ADRES3     7

#define TACH0      8
#define TACH1      9
#define TACH2     10
#define TACH3     11
#define MAX_CHNNL TACH3




// FUNCTION PROTOTYPES

unsigned int Calc_Checksum( unsigned char *ptr, unsigned char length );
void I2CBusState( void );
void Write_I2CSlave( void );
void ComposeBuffer( void );
void I2CBusState ( void );


// VARIABLES ( DEFINED HERE )

unsigned char slave_count, read_count;
unsigned char address_hold, address_offset;
unsigned char read_retry;
unsigned char write_count;

unsigned char I2CWriteState;
unsigned char index;
unsigned char i2cstate;
unsigned char *Write2Slave_Ptr;

bank1 unsigned char ReadStatBufFromSlave[OperChannelsPerSlave+8]; 
bank1 unsigned char *ReadFSlave_Ptr;

unsigned char WriteData2Slave[MaxChannelsPerSlave];

 
// FORMAT -> byte request count, functional code, checksum (repeats per each slave)
unsigned char WriteStatBuf2Slave[MaxNumberI2CSlaves * 3] = { 0x83,TEMP0,0x78, 0x83,TEMP0,0x76, 0x83,TEMP0,0x74,
                                                             0x83,TEMP0,0x72, 0x83,TEMP0,0x70, 0x83,TEMP0,0x6E,
                                                             0x83,TEMP0,0x6C, 0x83,TEMP0,0x6A, 0x83,TEMP0,0x68,
                                                             0x83,TEMP0,0x66, 0x83,TEMP0,0x64, 0x83,TEMP0,0x62 };


union {
	unsigned int error;
	unsigned int checksum;
	struct {
		unsigned char lobyte;
		unsigned char hibyte;
	} hold;
} temp;


union {
	unsigned int error_word;
	struct {
		unsigned char lobyte;
		unsigned char hibyte;
	} error;
} bus;


union {
	unsigned int error_word;
	struct {
		unsigned char lobyte;
		unsigned char hibyte;
	} error;
} comm;


union {
	unsigned int word;
	struct {
		unsigned char low;
		unsigned char high;
	} byte;
} checksum;



// VARIABLES ( DECLARED HERE / REFERENCE LINKAGE )

extern union events {
		unsigned int status;                   // entire status word
		struct bit_events {                    // structure with 8 bits
            unsigned int usart_tx         :1;  // flag indicating USART transmit event 
            unsigned int                  :1;  // 
			unsigned int i2c              :1;  // flag indicating I2C event
			unsigned int                  :1;  //
			unsigned int                  :1;  // 
            unsigned int write_state      :1;  // flag indicating write state entered
			unsigned int writes_done      :1;  // flag indicating that write state done 
            unsigned int                  :1;  // 
			unsigned int                  :1;  // 
			unsigned int                  :1;  // 
			unsigned int next_i2cslave    :1;  // flag indicating service next slave		
			unsigned int read_loop        :1;  // flag indicating read loop in progress  
			unsigned int read_start       :1;  // flag indicating read state entered
            unsigned int reads_done       :1;  // flag indicating that read state is done 
            unsigned int read_i2c         :1;  // flag indicating I2C read state  
            unsigned int                  :1;  //  
        } event;
} sflag;   


extern union i2c_error_events {
		unsigned char status;   
		struct error_events {                  // structure with 16 bits
            unsigned int slave_override    :1; // flag indicating 
            unsigned int retry_attempt     :1; // flag indicating to retry I2C comm
			unsigned int                   :1; // 
			unsigned int                   :1; //
			unsigned int                   :1; //  
			unsigned int overlimit         :1; // flag indicating byte is out-of-range
            unsigned int ack_error         :1; // flag indicating acknowledge error
            unsigned int bus_coll_error    :1; // flag indicating bus collision error
		} i2c;
} eflag;


// define I2C bus states
enum i2c_bus_states{ START =1, RESTART =2, STOP =3, SEND_ACK =4, SEND_NACK =5, GEN_CALL =6, 
                     READ =7, WRITE_DATA =8, WRITE_ADDRESS1 =9, WRITE_ADDRESS0 =10 };

const unsigned char ReadFSlaveI2CStates[] = {1,10,8,8,8,2,9,7,4,7,5,3,0}; //Pad with null state
const unsigned char Write2SlaveStates[] = {1,10,8,8,8,8,2,9,7,5,3,0}; //Pad with null state

const unsigned char *I2CState_Ptr;          // define pointer for accessing I2C states


// Slave Address defined here ( base  ---------------->   1,2,3,4, 5, 6, 7, 8, 9,10,11,12 
const unsigned char SlaveAddress[MaxNumberI2CSlaves+1] = {2,4,6,8,10,12,14,16,18,20,22,24,0};
