
/*  ******************************************************************
*                                                                    *
*                       Software License Agreement                   *
*                                                                    *
*   The software supplied herewith by Microchip Technology           *
*   Incorporated (the "Company") for its PICmicro Microcontroller   *
*   is intended and supplied to you, the Companys customer, for use *
*   solely and exclusively on Microchip PICmicro Microcontroller     *
*   products. The software is owned by the Company and/or its        *
*   supplier, and is protected under applicable copyright laws. All  *
*   rights are reserved. Any use in violation of the foregoing       *
*   restrictions may subject the user to criminal sanctions under    *
*   applicable laws, as well as to civil liability for the breach of *
*   the terms and conditions of this license.                        *
*                                                                    *
*   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           *
*   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    *
*   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    *
*   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     *
*   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  *
*   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  *
*                                                                    *
*********************************************************************/


/*********************************************************************
*                                                                    *
*           I2C Master and Slave Network using the PICmicro          *       
*                                                                    *
**********************************************************************
*                                                                    *
*   Filename:       init.c                                           *
*   Date:           06/09/2000                                       *
*   Revision:       1.00                                             *
*                                                                    *
*   Tools:          MPLAB 5.00.00                                    *
*                   Hi-Tech PIC C Compiler V7.85                     *
*                                                                    *
*   Author:         Richard L. Fischer                               *
*   Company:        Microchip Technology Incorporated                *
*                                                                    *
**********************************************************************
*                                                                    *
*   Files required:                                                  *
*                                                                    *
*                   pic.h                                            *
*                                                                    *
**********************************************************************
*                                                                    *
*   Notes: The routines within this file are required for            *
*          initializing the PICmicro peripherals and Ports.          *
*                                                                    *
*                                                                    *
*********************************************************************/

	#include <pic.h>                     // processor if/def file


#define FOSC         (16000000L)         // define external clock frequency
#define baud         19200               // define USART baud rate
#define i2c_bus_rate (400000L)           // define I2C bus rate




/*******************************************************************
********************************************************************/

void Init_Ports( void )
{
	OPTION &= 0b01111110;                //  

	PORTA = 0b000000;                    // set default pin drive states
	PORTB = 0b00000000;                  // set default pin drive states
	PORTC = 0b00000000;                  // set default pin drive states

	ADCON1 = 0b00000110;                 // ensure PortA is digital
	TRISA = 0b000000;                    // set PORTA as outputs 
	TRISB = 0b11110000;                  // RB0-RB3 outputs, RB4-RB7 inputs
	TRISC = 0b11011000;                  // 
}


void Init_Usart( void )
{
	unsigned long temp;                  // define auto type long variable
	/* calculate and set baud rate register for asynchronous mode */
	temp = 16UL * baud;

	TRISC |= 0b11000000;                 // ensure Rx and Tx are inputs (default)
	SPBRG = (int)(FOSC/temp) - 1;        // 9600 baud @ 16MHz
	TXSTA = 0b00100100;                  // enable Transmitter, BRGH = 1
	RCREG;                               // dummy read
	RCSTA = 0b10010000;                  // continuous receive, serial port enabled 
}


void Init_Ssp( void )
{
	TRISC |= 0b00011000;                 // ensure SDI and SD0 are inputs
	SSPIF = 0;                           // reset I2C based interrupt flag
	SSPCON2 = 0b00000000;                // ensure all state bits are reset
	SSPSTAT = 0b00000000;                // 
	SSPADD = (( FOSC / (4 * i2c_bus_rate) )) - 1; // initialize i2c bus rate
	SSPCON = 0b00111000;                 // Master I2C mode
}


void Init_Timer1( void )                 // set for 100mS intervals
{
	T1CON = 0b00110000;                  // 1:8 Prescale, T1OSCEN shut-off
	TMR1L = 0x60;                        // initialize TMR1 for 
	TMR1H = 0x3C;                        // 100mS intervals
	TMR1IF = 0;                          // reset Timer1 overflow flag
	TMR1ON = 1;                          // turn on Timer1 module	
}
