
/*********************************************************************
*                                                                    *
*                       Software License Agreement                   *
*                                                                    *
*   The software supplied herewith by Microchip Technology           *
*   Incorporated (the "Company") for its PICmicro Microcontroller   *
*   is intended and supplied to you, the Companys customer, for use *
*   solely and exclusively on Microchip PICmicro Microcontroller     *
*   products. The software is owned by the Company and/or its        *
*   supplier, and is protected under applicable copyright laws. All  *
*   rights are reserved. Any use in violation of the foregoing       *
*   restrictions may subject the user to criminal sanctions under    *
*   applicable laws, as well as to civil liability for the breach of *
*   the terms and conditions of this license.                        *
*                                                                    *
*   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           *
*   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    *
*   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    *
*   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     *
*   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  *
*   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  *
*                                                                    *
*********************************************************************/


/*********************************************************************
*                                                                    *
*           I2C Master and Slave Network using the PICmicro          *     
*                                                                    *
**********************************************************************
*                                                                    *
*   Filename:       mstri2c.c                                        *
*   Date:           06/09/2000                                       *
*   Revision:       1.00                                             *
*                                                                    *
*   Tools:          MPLAB 5.00.00                                    *
*                   Hi-Tech PIC C Compiler V7.85                     *
*                                                                    *
*   Author:         Richard L. Fischer                               *
*   Company:        Microchip Technology Incorporated                *
*                                                                    *
**********************************************************************
*                                                                    *
*   System files required:                                           *
*                                                                    *
*                   mstri2c.c                                        *
*                   i2c_comm.c                                       *
*                   init.c                                           *
*                   delay.c      (Hi-Tech file)                      *
*                                                                    *
*                   pic.h        (Hi-Tech file)                      *
*                   delay.h      (Hi-Tech file)                      *
*                   mstri2c.h                                        *
*                   i2c_comm.h                                       *
*                   cnfig87x.h                                       *
*                                                                    *
**********************************************************************
*                                                                    *
*   Notes:                                                           *
*	                                                                 *
*	Device Fosc -> 16.00MHz                                          *
*	WDT -> on                                                        *
*   Brownout -> on                                                   *
*   Powerup timer -> on                                              *
*   Code Protect -> off                                              *
*                                                                    *
*   Interrupt sources -                                              *
*                       1. USART based transmissions                 *
*                       2. I2C events (valid events)                 *
*                       3. I2C Bus Collision                         *
*                       4. Timer1 - 100mS intervals                  *
*                                                                    *
*                                                                    *
*                                                                    *
*   Memory Usage Map:                                                *
*                                                                    *
*   Program ROM   $0000 - $00BC  $00BD ( 189) words                  * 
*   Program ROM   $0587 - $07FF  $0279 ( 633) words                  *
*   Program ROM   $2007 - $2007  $0001 (   1) words                  *
*                                $0337 ( 823) words total Program ROM* 
*                                                                    * 
*                                                                    *
*   Bank 0 RAM    $0020 - $0075  $0056 (  86) bytes                  *
*   Bank 0 RAM    $007F - $007F  $0001 (   1) bytes                  *
*                                $0057 (  87) bytes total Bank 0 RAM *
*                                                                    *
*   Bank 1 RAM    $00A0 - $00AA  $000B (  11) bytes                  *
*   Bank 1 RAM    $00FF - $00FF  $0001 (   1) bytes                  *
*                                $000C (  12) bytes total Bank 1 RAM *
*                                                                    *
*********************************************************************/

	#include <pic.h>                     // processor if/def file
	#include "cnfig87x.h"                // configuration word definitions
	#include "mstri2c.h"                 //    
	#include "c:\ht-pic\samples\delay.h" //


	__CONFIG   ( CONBLANK & CP_OFF & DEBUG_OFF & WRT_ENABLE_OFF & CPD_OFF & 
                 LVP_OFF & BODEN_ON & PWRTE_ON & WDT_ON & HS_OSC );




/*******************************************************************
  MAIN PROGRAM BEGINS HERE
********************************************************************/

void main(void)
{
/* Initialization is done here */
	Init_Usart();                        // initialize USART peripheral
	Init_Ports();                        // initialize Ports
	Init_Ssp();                          // initialize SSP module
	Init_Timer1();                       // initialize TMR1 peripheral
 
/* Interrupts are enabled here */
	TMR1IE = 1;                          // enable TMR1 Overflow interrupt
	BCLIE = 1;                           // enable bus collision interrupt
	SSPIE = 1;                           // enable I2C based interrupts

	sflag.status = 0x0000;               // ensure all event flags are reset	
	eflag.status = 0x00;                 // ensure all event flags are reset

	PEIE = 1;                            // enable peripheral interrupts
    ei();                                // enable global interrupts
 

    for ( ;; )                           // infinite loop
    { 
	  CLRWDT();                          // reset WDT

//------------------------------------------------------------------------
//   Will execute these statements if Master wants to read from Slave I2C
//------------------------------------------------------------------------
	  if ( sflag.event.read_i2c && sflag.event.i2c ) // test if read I2C is active
	  {
		if ( !sflag.event.reads_done )   // test if more reads are needed
		{
		  DelayUs( 400 );                // short delay between events
                                         // to allow for slow FOSC on Slave
		  Service_I2CSlave();            // Service I2C slave device(s)
		}

		if ( sflag.event.reads_done )    // test if that was last read
		{
		  sflag.status &= 0x00F0;        // reset all I2C event flags
		  eflag.status = 0x00;           // reset all error event flags		
  		  TMR1ON = 1;                    // turn on Timer1 module	
		  TMR1IE = 1;                    // re-enable Timer1 interrupt 
		}
	  }
	}
}



//------------------------------------------------------------------------
//   Will evaluate these conditional statements on an interrupt basis 
//------------------------------------------------------------------------
void interrupt piv( void )
{
	if ( SSPIE && SSPIF )                // test for I2C event completion
	{
 	  SSPIF = 0;                         // reset I2C based interrupt flag
	  sflag.event.i2c = 1;               // set I2C event service flag	
	  PORTB ^= 0b00000001;               // ***** test purposes only *****
	}

	else if ( BCLIE && BCLIF )           // test for bus collision
	{
	  eflag.i2c.bus_coll_error = 1;      // set bus collision flag error
	  sflag.event.i2c = 1;               // set I2C event service flag
	  BCLIF = 0;                         // reset bus collision interrupt flag
	  PORTB ^= 0b00000010;               // ***** test purposes only *****
	}

	else if ( TXIE && TXIF )             // test if USART based transmit interrupt
	{
	  if ( index < MaxLength2PC )        // is all data sent ?
	  {
		TXREG = ReadStatBufFromSlave[index]; // send another byte out
 		index++;                         // increment array index
	  }
	  else
	  {
		sflag.event.usart_tx = 0;        // reset USART TX in progress flag
		TXIE = 0;                        // disable transmit interrupt
		SSPIE = 1;                       // enable I2C based interrupts
		index = 0x00;                    // reset array index
	  }
	}

	else if ( TMR1IE && TMR1IF )         // test for valid TMR1 interrupt
	{
	  sflag.event.read_i2c = 1;          // set 100mS event service flag
  	  sflag.event.i2c = 1;               // set I2C event service flag
	  PORTB &= 0b00001110;               // ***** test purposes only *****  
	  PORTB ^= 0b00001000;               // ***** test purposes only *****
	  TMR1ON = 0;                        // turn off Timer1 module		
	  TMR1IF = 0;                        // reset TMR1 rollover interrupt flag
	  TMR1IE = 0;                        // disable TMR1 based interrupt
	  TMR1L += 0x60;                     // re-initialize TMR1 for 
	  TMR1H = 0x3C;                      // 100mS intervals
	}
}

